import kotlinx.html.InputType
import kotlinx.html.js.onChangeFunction
import kotlinx.html.js.onClickFunction
import kotlinx.serialization.decodeFromString
import kotlinx.serialization.json.Json
import org.w3c.dom.HTMLInputElement
import org.w3c.xhr.XMLHttpRequest
import react.*
import react.dom.option
import schema.JsonSchema
import schema.JsonType
import styled.*

external interface SchemaPartProps : Props {
    var source: String?
    var name: String
    var schema: JsonSchema?
    var required: Boolean
}

data class SchemaPartState(
    var source: String?,
    var name: String,
    var schema: JsonSchema?,
    var required: Boolean,
    var arrayItems: MutableList<JsonSchema> = mutableListOf(),
    var mapItems: MutableMap<String, JsonSchema> = mutableMapOf(),
    var nextMapItem: String= "",
): State

class SchemaPart(props: SchemaPartProps) : RComponent<SchemaPartProps, SchemaPartState>(props) {

    init {
        this.state = SchemaPartState(
            source = props.source,
            name = props.name,
            schema = props.schema,
            required = props.required,
        )

        val json = Json {
            ignoreUnknownKeys = true
            prettyPrint = true
        }

        val xmlHttp = XMLHttpRequest()
        if (state.source != null) {
            xmlHttp.open("GET", "${props.source}")
            xmlHttp.onload = {
                val schema: JsonSchema = json.decodeFromString(xmlHttp.responseText)
                setState {
                    this.schema = schema
                }
            }

            xmlHttp.send()
        }
    }

    override fun RBuilder.render() {
        if (state.name == "\$schema")
            return

        styledDiv {
            css {
                +SchemaPartStyles.schemaPart
            }

            if (state.schema?.type == JsonType.NUMBER) {
                styledDiv {
                    if (state.required)
                        +"*"

                    +state.name

                    styledInput(InputType.number) {

                    }
                }
                return@styledDiv
            }

            if (state.schema?.type == JsonType.STRING) {
                styledDiv {
                    if (state.required)
                        +"*"

                    +state.name

                    if (state.schema?.enum?.isNotEmpty() == true) {
                        styledSelect {
                            for (opt in state.schema!!.enum) {
                                option("", opt)
                            }
                        }
                    } else {
                        styledInput(InputType.text) {

                        }
                    }
                }
                return@styledDiv
            }

            if (state.schema?.type == JsonType.BOOLEAN) {
                styledDiv {
                    if (state.required)
                        +"*"

                    +state.name

                    styledInput(InputType.checkBox) {

                    }
                }
                return@styledDiv
            }

            styledSpan {
                css {
                    +SchemaPartStyles.schemaName
                }
                if (state.required)
                    +"*"

                +state.name
            }

            styledDiv {
                var type = state.schema?.type
                if (type == null || type == JsonType.UNKNOWN)
                    type = state.schema?.conditionIf?.type

                if (!(type == JsonType.OBJECT || type == JsonType.ARRAY))
                    +"Type: $type"
            }

            if (state.schema != null) {
                // Object properties
                for (property in state.schema!!.properties) {
                    child(SchemaPart::class) {
                        attrs {
                            name = property.key
                            schema = property.value
                            required = state.schema?.required?.contains(property.key) ?: false
                        }
                    }
                }

                // Array entries
                if (state.schema?.type == JsonType.ARRAY) {
                    styledButton {
                        +"+"
                        attrs.onClickFunction = {
                            setState {
                                arrayItems.add(state.schema!!.items!!)
                            }
                        }
                    }
                }

                // Object map elements
                if (state.schema?.additionalProperties != null) {
                    styledInput {
                        attrs.onChangeFunction = {
                            state.nextMapItem = (it.target as HTMLInputElement).value
                        }
                    }
                    styledButton {
                        +"+"
                        attrs.onClickFunction = {
                            setState {
                                mapItems[state.nextMapItem] = state.schema!!.additionalProperties!!
                            }
                        }
                    }
                }
            }

            for (child in state.arrayItems) {
                child(SchemaPart::class) {
                    attrs {
                        name = "Item"
                        schema = child
                    }
                }
            }

            for (child in state.mapItems) {
                child(SchemaPart::class) {
                    attrs {
                        name = child.key
                        schema = child.value
                    }
                }
            }
        }
//        styledDiv {
//            css {
//                +WelcomeStyles.textContainer
//            }
//            +"Hello, ${state.name}"
//        }
//        styledInput {
//            css {
//                +WelcomeStyles.textInput
//            }
//            attrs {
//                type = InputType.text
//                value = state.name
//                onChangeFunction = { event ->
//                    setState(
//                        WelcomeState(name = (event.target as HTMLInputElement).value)
//                    )
//                }
//            }
//        }
    }
}
