package schema

import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import kotlinx.serialization.json.JsonObject

@Serializable
data class JsonSchema(
    @SerialName("\$schema")
    val schema: String = "",
    val type: JsonType = JsonType.UNKNOWN,
    val properties: Map<String, JsonSchema> = mapOf(),
    val additionalProperties: JsonSchema? = null,
    val definitions: Map<String, JsonObject> = mapOf(),
    val required: List<String> = listOf(),
    val items: JsonSchema? = null,
    val description: String = "",
    val enum: List<String> = listOf(),
    @SerialName("if") val conditionIf: JsonSchema? = null,
    @SerialName("then") val conditionThen: JsonSchema? = null,
    @SerialName("else") val conditionElse: JsonSchema? = null,
)

@Serializable
enum class JsonType {
    @SerialName("unknown") UNKNOWN,
    @SerialName("string") STRING,
    @SerialName("number") NUMBER,
    @SerialName("boolean") BOOLEAN,
    @SerialName("object") OBJECT,
    @SerialName("array") ARRAY,
    @SerialName("null") NULL,

}
